<?php
/**
 * Map To API Page
 *
 * @package Wp2Leads/Partials
 * @version 1.0.1.7
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

if ( $activeMap ) {

	$api         = unserialize( $activeMap->api );
    $mapping         = unserialize( $activeMap->mapping );
    $info        = unserialize( $activeMap->info );

	$is_map_have_results = Wp2leads_MapsActivation::is_map_active($_GET['active_mapping']);
	$is_plugins_ok = true;

	if ( $info['serverId'] ) {
		$plugins = new Wp2leads_RequiredPlugins();
		if ( count( $plugins->check_map_plugins($info['serverId']) ) ) {

			$is_plugins_ok = false;

		}
	}
	
	if ( !empty($_GET['start_step']) ) {
		echo '<input type="hidden" id="start_step" value="' . $_GET['start_step'] . '">';
	}
} ?>

<h2><?php _e( 'Map to API', 'wp2leads' ) ?></h2>

<hr>

<?php
if ( !$activeMap ) {
    ?>
    <div class="map-to-api">
        <?php include_once 'wp2leads-admin-map_to_api-no-active-map.php'; ?>
    </div>
    <?php
} else if ( !$is_plugins_ok || !$is_map_have_results ) {
	?>
    <div class="map-to-api">
		<?php
			if ( !$is_map_have_results ) {
				_e('This map is not active:', 'wp2leads');
				echo ' ' . Wp2leads_MapsActivation::get_map_error_message($_GET['active_mapping']);
			}

			if ( !$is_plugins_ok ) {

				echo '<br>';
				_e( 'The next plugins are required for this map: ', 'wp2leads' );
				echo '<ul>';
				foreach ( $plugins->check_map_plugins($info['serverId']) as $plugin ) {
					echo '<li> - ' . $plugin['label'] . '</li>';
				}
				echo '</ul>';
				echo '<button id="installPluginsMapToAPI" class="button button-green" data-id="' . $info['serverId'] . '">' . __('Install Plugin(s)', 'wp2leads') . '</button>';

				echo '<div class="map-to-api-install-plugins" style="display: none;">' . __('Installing...', 'wp2leads') . '</div>';
			}  ?>
    </div>
    <?php
} else {
    $activeMapId = '';
    $is_load_data = false;

    if ( $_GET['tab'] == 'map_to_api' && isset( $_GET['active_mapping'] ) ) {
        global $wpdb;
        $table       = $wpdb->get_row( sprintf( "SELECT * FROM %s WHERE id=%d", $wpdb->prefix . "wp2l_maps", (int) $_GET['active_mapping'] ) );

        $activeMapId = $_GET['active_mapping'];

        $tablesToCheck = array();
        array_push($tablesToCheck, $mapping['from']);

        if ( !empty( $mapping['joins'] ) ) {
            foreach ( $mapping['joins'] as $join ) {
                array_push( $tablesToCheck, $join['joinTable'] );
            }
        }

        // compare existing tables against the map tables
        $mapValid = true;

        foreach ($tablesToCheck as $checkMe) {
            if ( !in_array( MapsModel::unindexed_table_name($checkMe), $tables ) ) {
                $mapValid = false;
                break;
            }
        }
    }

    $connector = new Wp2leads_KlicktippConnector();
    $logged_in = $connector->login();
    $logged_in_error = $connector->get_last_connector_error();

    if ( $logged_in ) {

		// check losted tags
		if (!empty($api['losted_manually_selected_tags'])) {

			foreach ($api['losted_manually_selected_tags'] as $t) {
				$result = $connector->tag_create(trim($t));
				if ($result) {
					$code = $result;
				} else {
					$code = array_search(trim($t), $connector->tag_index());
				}

				if ($code) {
					$api['manually_selected_tags']['tag_ids'][] = $code;
				}
			}

			unset($api['losted_manually_selected_tags']);
			MapsModel::updateMapCell($activeMap->id, 'api', serialize($api));
		}

		if (!empty($api['losted_tags'])) {
			$admin_ajax = new Wp2leads_Admin_Ajax();
			$admin_ajax->add_recommended_klick_tip_tags($api['losted_tags']);
			unset($api['losted_tags']);
			MapsModel::updateMapCell($activeMap->id, 'api', serialize($api));
		}

        $fields = $connector->field_index();
        $tags   = (array) $connector->tag_index();
        asort( $tags, SORT_STRING | SORT_FLAG_CASE | SORT_NATURAL );
        $optins = $connector->subscription_process_index();
    }

    $is_transfer_allowed = Wp2leads_License::is_map_transfer_allowed( (int) $_GET['active_mapping'] );
    $is_initial_settings_done = !empty($info['initial_settings']);
    // $is_initial_settings_done = true;

    $is_load_data = $is_initial_settings_done && $mapValid && $logged_in;

    if ( !$is_transfer_allowed && $mapValid ) {
        $kt_limitation = KlickTippManager::get_initial_kt_limitation();

        if (!$kt_limitation) {
            ?>
            <div class="wp2leads-notice wp2leads-notice-warning">
                <h4><?php _e( 'Transfer Map only in paid versions', 'wp2leads' ) ?></h4>

                <p><?php _e( 'Please upgrade to Professional Version to transfer user via OWN created maps. <a href="https://wp2leads-for-klick-tipp.com/?p=7" target="_blank">Buy it</a>, or <button class="button button-small" id="license-modal-open">Enter a license</button>', 'wp2leads' ) ?></p>
            </div>
            <?php
        }
    }

    ?>

	<?php if (isset($api['remove_notice'])) { ?>
		<input type="hidden" id="remove_notice" value="<?php echo $api['remove_notice']; ?>">
	<?php } ?>

    <div id="map-to-api-message-in-progress__holder"></div>

    <div class="map-to-api">
        <?php
        $cron_status   = '';
        $cron_title    = __( 'Cron not set up', 'wp2leads' );
        $cron_checked = '';
        $cron_selected = '';

        $cron_available = false;

        $cron_active = false;

        if (!empty($decodedMap['dateTime']) && is_array($decodedMap['dateTime'])) {
            $cron_maps = Wp2LeadsCron::getScheduledMaps();
            $cron_fields = array();

            $cron_available = true;

            if ( empty( $cron_maps[ 'map_' . $activeMapId ] ) || (!empty($cron_maps[ 'map_' . $activeMapId ]['status_to_change']) && 'remove_cron_schedule' === $cron_maps[ 'map_' . $activeMapId ]['status_to_change']) ) {
                $cron_status   = '';
                $cron_title    = __( 'Cron not set up', 'wp2leads' );
                $cron_checked  = '';
                $cron_selected = '';
            } else {
                $cron_status   = ' disabled';
                $cron_title    = __( 'Cron disabled', 'wp2leads' );
                $cron_checked  = '';
                $cron_selected = '';

                if ( ! empty( $cron_maps[ 'map_' . $activeMapId ]['status'] ) && empty($cron_maps[ 'map_' . $activeMapId ]['status_to_change']) ) {
                    $cron_status  = ' active';
                    $cron_title   = __( 'Cron enabled', 'wp2leads' );
                    $cron_checked = 'checked';

                    $cron_active = true;
                }

                if ( ! empty( $cron_maps[ 'map_' . $activeMapId ]['date_base'] ) ) {
                    if ( ! is_array( $cron_maps[ 'map_' . $activeMapId ]['date_base'] ) ) {
                        $cron_selected = htmlspecialchars( json_encode( array( $cron_maps[ 'map_' . $activeMapId ]['date_base'] ) ) );
                        $cron_fields = array( $cron_maps[ 'map_' . $activeMapId ]['date_base'] );
                    } else {
                        $cron_selected = htmlspecialchars( json_encode( $cron_maps[ 'map_' . $activeMapId ]['date_base'], JSON_HEX_QUOT ) );
                        $cron_fields = $cron_maps[ 'map_' . $activeMapId ]['date_base'];
                    }
                }
            }

            if ( ! $is_transfer_allowed ) {
                $cron_status = '';
            }
        }

        $module_available = false;
        $module_active = false;

        if (!empty($mapping["transferModule"])) {
            $transfer_modules = Wp2leads_Transfer_Modules::get_transfer_modules_class_names();

            if (!empty($transfer_modules[$mapping["transferModule"]])) {
                $transfer_module = $transfer_modules[$mapping["transferModule"]];
                $class_name = $transfer_module;
                $module_label = $class_name::get_label();
                $module_description = $class_name::get_description();
                $existed_modules_map = Wp2leads_Transfer_Modules::get_modules_map();
                $module_enabled = !empty($existed_modules_map[$mapping["transferModule"]][$activeMapId]) ? ' checked' : '';

                $module_available = true;

                if (!empty($existed_modules_map[$mapping["transferModule"]][$activeMapId])) {
                    $module_active = true;
                }
            }
        }

        $all_columns      = $decodedMap['selects'];
        $excluded_columns = ! empty( $decodedMap['excludes'] ) ? $decodedMap['excludes'] : false;

        if ( $excluded_columns ) {
            $map_columns = array_diff( $all_columns, $excluded_columns );
        } else {
            $map_columns = $all_columns;
        }

		$losted_name =  !empty( $api['losted_name'] ) ? $api['losted_name'] : '';
		if ( !empty( $api['losted_name'] ) && is_array($api['losted_name']) ) $losted_name = $api['losted_name'][0];

        ?>

        <input type="hidden" class="mapping"
               value='<?php echo ! empty( $api ) ? json_encode( $api ) : ''; ?>'
               data-prev_value=''
               data-new_value=''
        >
		<input type="hidden" class="losted_name"
			value='<?php echo $losted_name; ?>'
		>

        <div id="map-to-api__container">
            <div id="map-to-api__header">
                <h3 class="title">
                    <?php echo MapBuilderManager::get_clock_icon_for_map($activeMapId); ?>
                    <?php echo stripslashes( $activeMap->name ) ?>
                </h3>

                <div class="buttons-holder">
                    <?php

                    if ($is_initial_settings_done && $mapValid) {
                        ?>
                        <button id="btnSaveMapToApi" class="button button-primary">
                            <?php echo __( 'Save map', 'wp2leads' ); ?>
                        </button>

                        <button id="btnUpdateMapToApi" class="button button-primary">
                            <?php echo __( 'Save and Exit', 'wp2leads' ); ?>
                        </button>
                        <?php
                    }

                    ?>
                    <a href="?page=wp2l-admin&tab=map_to_api" class="button button-primary">
                        <?php _e( 'Exit Map', 'wp2leads' ) ?>
                    </a>

                    <?php
                    $class = 'secondary';
                    $args  = array( 'disabled' => 'disabled' );

                    if ( $is_transfer_allowed ) {
                        $class = 'primary';
                        $args  = array(
                            'data-active-map'    => $activeMapId,
                            'data-cron-checked'  => $cron_checked,
                            'data-cron-selected' => $cron_selected,
                        );
                    }
                    ?>
                </div>
            </div>

            <div id="map-to-api__body" class="fixed-panel">
                <div id="map-to-api__map-list">
                    <?php require_once dirname( __FILE__ ) . '/wp2leads-admin-runner-map-list.php'; ?>
                </div>

                <?php
                if (!$mapValid) {
                    ?>
                    <div id="map-to-api__full-column">
                        <div class="notice notice-error inline" style="margin-top:0;margin-bottom:0;">
                            <p><?php _e('This saved map refers to a table no longer present in the database.', 'wp2leads') ?></p>
                        </div>
                    </div>
                    <?php
                } else {
                    ?>
                    <div id="map-to-api__left-column" class="panel-hidden">
						<div class="map-to-api-toggle-panel">
							<span class="dashicons dashicons-admin-collapse"></span>
							<span class="tp-button"><?php _e('DB Entries'); ?></span>
						</div>
                        <div class="map2api_body">
                            <?php
                            if (!$logged_in) {
                                ?>
                                <div class="no_available_options">
                                    <p><?php _e('You must be logged in to KT account to continue working with this map.', 'wp2leads') ?></p>
                                </div>
                                <?php
                            } elseif (!$is_initial_settings_done) {
                                ?>
                                <div class="no_available_options">
                                    <p><?php _e('Please make initial settings to continue working with this map.', 'wp2leads') ?></p>
                                </div>
                                <?php
                            } else {
                                ?>
                                <div class="available_options">
                                    <?php
                                    include_once 'ajax/available_options_page_empty.php';
                                    ?>
                                </div>
                                <?php
                            }
                            ?>
                        </div>
                    </div>

                    <div id="map-to-api__right-column">
                        <div class="map2api_side">
                            <div class="api_fields_container<?php echo $is_load_data ? '' : ' no-data-load' ?>">
                                <?php
                                if ( $logged_in ) {
                                    ?>
                                    <!-- Initial Settings Section - Start -->
                                    <div class="accordion-group">
                                        <?php include_once 'wp2leads-admin-map_to_api-initial-settings.php'; ?>
                                    </div>
                                    <!-- Initial Settings Section - End -->

                                    <!-- Overview Section - Start -->
                                    <div class="accordion-group">
                                        <?php include_once 'wp2leads-admin-map_to_api-overview.php'; ?>
                                    </div>
                                    <!-- Overview Section - End -->

                                    <div class="accordion-group">
                                        <!-- Optins Section - Start -->
                                        <?php include_once 'wp2leads-admin-map_to_api-optins.php'; ?>
                                        <!-- Optins Section - End -->

                                        <!-- API Fields Section - Start -->
                                        <?php include_once 'wp2leads-admin-map_to_api-fields.php'; ?>
                                        <!-- API Fields Section - End -->

                                        <hr>

                                        <h3 style="margin-bottom: 10px"><?php _e( 'Tags', 'wp2leads' ) ?></h3>

                                        <!-- Automatic Tags Section - Start -->
                                        <?php include_once 'wp2leads-admin-map_to_api-auto-tags.php'; ?>
                                        <!-- Automatic Tags Section - End -->

                                        <!-- Add Tags Section - Start -->
                                        <?php include_once 'wp2leads-admin-map_to_api-add-tags.php'; ?>
                                        <!-- Add Tags Section - End -->

                                        <!-- Untag Tags Section - Start -->
                                        <?php include_once 'wp2leads-admin-map_to_api-untag-tags.php'; ?>
                                        <!-- Untag Tags Section - End -->
                                    </div>
                                    <?php
                                } else { // KT Logged In false
                                    if ($logged_in_error && ('no_login_data' === $logged_in_error['error'] || 'login_failed' === $logged_in_error['error'])) {
                                        ?>
                                        <div class="accordion-group">
                                            <?php include_once 'wp2leads-admin-map_to_api-no-login-data.php'; ?>
                                        </div>
                                        <?php
                                    }
                                }
                                ?>
                            </div>
                        </div>
                    </div>
                    <?php
                }
                ?>
            </div>
        </div>
    </div>
    <?php
}
